"""Auto-generated file, do not edit by hand. NO metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_NO = PhoneMetadata(id='NO', country_code=47, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:0|[2-9]\\d{3})\\d{4}', possible_length=(5, 8)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:2[1-4]|3[1-3578]|5[1-35-7]|6[1-4679]|7[0-8])\\d{6}', example_number='21234567', possible_length=(8,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:4[015-8]|9\\d)\\d{6}', example_number='40612345', possible_length=(8,)),
    toll_free=PhoneNumberDesc(national_number_pattern='80[01]\\d{5}', example_number='80012345', possible_length=(8,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='82[09]\\d{5}', example_number='82012345', possible_length=(8,)),
    shared_cost=PhoneNumberDesc(national_number_pattern='810(?:0[0-6]|[2-8]\\d)\\d{3}', example_number='81021234', possible_length=(8,)),
    personal_number=PhoneNumberDesc(national_number_pattern='880\\d{5}', example_number='88012345', possible_length=(8,)),
    voip=PhoneNumberDesc(national_number_pattern='85[0-5]\\d{5}', example_number='85012345', possible_length=(8,)),
    uan=PhoneNumberDesc(national_number_pattern='(?:0[235-9]|81(?:0(?:0[7-9]|1\\d)|5\\d\\d))\\d{3}', example_number='02000', possible_length=(5, 8)),
    voicemail=PhoneNumberDesc(national_number_pattern='81[23]\\d{5}', example_number='81212345', possible_length=(8,)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['8']),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[2-79]'])],
    main_country_for_code=True,
    leading_digits='[02-689]|7[0-8]',
    mobile_number_portable_region=True)



"""Auto-generated file, do not edit by hand. BE metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_BE = PhoneMetadata(id='BE', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[1-9]\\d\\d(?:\\d(?:\\d{2})?)?', possible_length=(3, 4, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='1(?:0[0-35-8]|1[0269]|7(?:12|77)|813)|(?:116|8)\\d{3}', example_number='100', possible_length=(3, 4, 6)),
    premium_rate=PhoneNumberDesc(national_number_pattern='1(?:2[03]|40)4|(?:1(?:[24]1|3[01])|[2-79]\\d\\d)\\d', example_number='1204', possible_length=(4,)),
    emergency=PhoneNumberDesc(national_number_pattern='1(?:0[01]|12)', example_number='100', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:0[0-8]|16117|2(?:12|3[0-24])|313|414|5(?:1[05]|5[15]|66|95)|6(?:1[167]|36|6[16])|7(?:[07][017]|1[27-9]|22|33|65)|81[39])|[2-9]\\d{3}|11[02679]|1(?:1600|45)0|1(?:[2-4]9|78)9|1[2-4]0[47]', example_number='100', possible_length=(3, 4, 6)),
    sms_services=PhoneNumberDesc(national_number_pattern='[2-9]\\d{3}', example_number='2000', possible_length=(4,)),
    short_data=True)



"""Auto-generated file, do not edit by hand. LU metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_LU = PhoneMetadata(id='LU', country_code=352, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='35[013-9]\\d{4,8}|6\\d{8}|35\\d{2,4}|(?:[2457-9]\\d|3[0-46-9])\\d{2,9}', possible_length=(4, 5, 6, 7, 8, 9, 10, 11)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:35[013-9]|80[2-9]|90[89])\\d{1,8}|(?:2[2-9]|3[0-46-9]|[457]\\d|8[13-9]|9[2-579])\\d{2,9}', example_number='27123456', possible_length=(4, 5, 6, 7, 8, 9, 10, 11)),
    mobile=PhoneNumberDesc(national_number_pattern='6(?:[26][18]|5[1568]|7[189]|81|9[128])\\d{6}', example_number='628123456', possible_length=(9,)),
    toll_free=PhoneNumberDesc(national_number_pattern='800\\d{5}', example_number='80012345', possible_length=(8,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='90[015]\\d{5}', example_number='90012345', possible_length=(8,)),
    shared_cost=PhoneNumberDesc(national_number_pattern='801\\d{5}', example_number='80112345', possible_length=(8,)),
    voip=PhoneNumberDesc(national_number_pattern='20(?:1\\d{5}|[2-689]\\d{1,7})', example_number='20201234', possible_length=(4, 5, 6, 7, 8, 9, 10)),
    national_prefix_for_parsing='(15(?:0[06]|1[12]|[35]5|4[04]|6[26]|77|88|99)\\d)',
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{3})', format='\\1 \\2', leading_digits_pattern=['2(?:0[2-689]|[2-9])|[3-57]|8(?:0[2-9]|[13-9])|9(?:0[89]|[2-579])'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3', leading_digits_pattern=['2(?:0[2-689]|[2-9])|[3-57]|8(?:0[2-9]|[13-9])|9(?:0[89]|[2-579])'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['20[2-689]'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{1,2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['20'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{1,5})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[3-57]|8[13-9]|9(?:0[89]|[2-579])|(?:2|80)[2-9]'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['80[01]|90[015]'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{3})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['20'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['6'], domestic_carrier_code_formatting_rule='$CC \\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{1,2})', format='\\1 \\2 \\3 \\4 \\5', leading_digits_pattern=['20'], domestic_carrier_code_formatting_rule='$CC \\1')],
    mobile_number_portable_region=True)



from collections.abc import Callable
from collections.abc import Iterator
from re import Pattern

from .util import U_EMPTY_STRING, U_DASH, U_SEMICOLON, U_SLASH, U_X_LOWER, U_X_UPPER, U_PERCENT
from .phonenumberutil import NON_DIGITS_PATTERN
from .phonemetadata import NumberFormat
from .phonenumber import PhoneNumber
from .util import UnicodeMixin

def _limit(lower: int, upper: int) -> str: ...

_OPENING_PARENS: str
_CLOSING_PARENS: str
_NON_PARENS: str
_BRACKET_PAIR_LIMIT: str
_MATCHING_BRACKETS: Pattern[str]
_LEAD_LIMIT: str
_PUNCTUATION_LIMIT: str
_DIGIT_BLOCK_LIMIT: int
_BLOCK_LIMIT: str
_PUNCTUATION: str
_DIGIT_SEQUENCE: str
_LEAD_CLASS_CHARS: str
_LEAD_CLASS: str
_LEAD_PATTERN: Pattern[str]
_PATTERN: Pattern[str]
_PUB_PAGES: Pattern[str]
_SLASH_SEPARATED_DATES: Pattern[str]
_TIME_STAMPS: Pattern[str]
_TIME_STAMPS_SUFFIX: Pattern[str]
_INNER_MATCHES: tuple[Pattern[str], ...]

class Leniency:
    POSSIBLE: int
    VALID: int
    STRICT_GROUPING: int
    EXACT_GROUPING: int

def _verify(leniency: int, numobj: PhoneNumber, candidate: str, matcher: PhoneNumberMatcher) -> bool: ...
def _verify_strict_grouping(numobj: PhoneNumber, candidate: str, matcher: PhoneNumberMatcher) -> bool: ...
def _all_number_groups_remain_grouped(numobj: PhoneNumber, normalized_candidate: str, formatted_number_groups: list[str]) -> bool: ...
def _verify_exact_grouping(numobj: PhoneNumber, candidate: str, matcher: PhoneNumberMatcher) -> bool: ...
def _all_number_groups_are_exactly_present(numobj: PhoneNumber, normalized_candidate: str, formatted_number_groups: list[str]) -> bool: ...
def _get_national_number_groups_without_pattern(numobj: PhoneNumber) -> list[str]: ...
def _get_national_number_groups(numobj: PhoneNumber, formatting_pattern: NumberFormat) -> list[str]: ...
def _contains_more_than_one_slash_in_national_number(numobj: PhoneNumber, candidate: str) -> bool: ...
def _contains_only_valid_x_chars(numobj: PhoneNumber, candidate: str) -> bool: ...
def _is_national_prefix_present_if_required(numobj: PhoneNumber) -> bool: ...

class PhoneNumberMatcher:
    _NOT_READY: int
    _READY: int
    _DONE: int
    text: str
    preferred_region: str | None
    leniency: int
    _max_tries: int
    _min_candidate_length: int
    _state: int
    _last_match: PhoneNumberMatch | None
    _search_index: int
    def __init__(self, text: str | None, region: str | None, leniency: int = ..., max_tries: int = ..., min_candidate_length: int = ...) -> None: ...
    def _find(self, index: int) -> PhoneNumberMatch | None: ...
    def _trim_after_first_match(self, pattern: Pattern[str], candidate: str) -> str: ...
    @classmethod
    def _is_latin_letter(cls, letter: str) -> bool: ...
    @classmethod
    def _is_invalid_punctuation_symbol(cls, character: str) -> bool: ...
    def _extract_match(self, candidate: str, offset: int) -> PhoneNumberMatch | None: ...
    def _extract_inner_match(self, candidate: str, offset: int) -> PhoneNumberMatch | None: ...
    def _parse_and_verify(self, candidate: str, offset: int) -> PhoneNumberMatch | None: ...
    def _check_number_grouping_is_valid(self, numobj: PhoneNumber, candidate: str, checker: Callable[[PhoneNumber, str, list[str]], bool]) -> bool: ...
    def has_next(self) -> bool: ...
    def next(self) -> PhoneNumberMatch: ...
    def __iter__(self) -> Iterator[PhoneNumberMatch]: ...

class PhoneNumberMatch(UnicodeMixin):
    start: int
    raw_string: str
    end: int
    number: PhoneNumber
    def __init__(self, start: int, raw_string: str, numobj: PhoneNumber) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __repr__(self) -> str: ...
    def __unicode__(self) -> str: ...



"""Auto-generated file, do not edit by hand. GP metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_GP = PhoneMetadata(id='GP', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[13]\\d(?:\\d\\d(?:\\d{2})?)?', possible_length=(2, 4, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='1[578]|3(?:00|1[0-689])\\d', example_number='15', possible_length=(2, 4)),
    premium_rate=PhoneNumberDesc(national_number_pattern='3[2469]\\d\\d', example_number='3200', possible_length=(4,)),
    emergency=PhoneNumberDesc(national_number_pattern='1[578]', example_number='15', possible_length=(2,)),
    short_code=PhoneNumberDesc(national_number_pattern='1[578]|3(?:00[0-79]|1[0-689]\\d)|(?:118[02-9]|3[2469])\\d\\d', example_number='15', possible_length=(2, 4, 6)),
    short_data=True)



"""Auto-generated file, do not edit by hand. GF metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_GF = PhoneMetadata(id='GF', country_code=594, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:694\\d|7093)\\d{5}|(?:59|[89]\\d)\\d{7}', possible_length=(9,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:59(?:4(?:[02-49]\\d|1[0-5]|5[6-9]|6[0-3]|80)|88\\d)|80[6-9]\\d\\d)\\d{4}', example_number='594101234', possible_length=(9,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:694(?:[0-249]\\d|3[0-8])|7093[0-3])\\d{4}', example_number='694201234', possible_length=(9,)),
    toll_free=PhoneNumberDesc(national_number_pattern='80[0-5]\\d{6}', example_number='800012345', possible_length=(9,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='8[129]\\d{7}', example_number='890123456', possible_length=(9,)),
    voip=PhoneNumberDesc(national_number_pattern='9(?:(?:396|76\\d)\\d|476[0-6])\\d{4}', example_number='976012345', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[5-7]|80[6-9]|9[47]'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[89]'], national_prefix_formatting_rule='0\\1')],
    mobile_number_portable_region=True)



from .util import U_EMPTY_STRING
from .phonemetadata import PhoneNumberDesc
from .phonenumber import PhoneNumber

SUPPORTED_SHORT_REGIONS: list[str]
_REGIONS_WHERE_EMERGENCY_NUMBERS_MUST_BE_EXACT: set[str]

class ShortNumberCost:
    TOLL_FREE: int
    STANDARD_RATE: int
    PREMIUM_RATE: int
    UNKNOWN_COST: int

def _region_dialing_from_matches_number(numobj: PhoneNumber, region_dialing_from: str) -> bool: ...
def is_possible_short_number_for_region(short_numobj: PhoneNumber, region_dialing_from: str) -> bool: ...
def is_possible_short_number(numobj: PhoneNumber) -> bool: ...
def is_valid_short_number_for_region(short_numobj: PhoneNumber, region_dialing_from: str) -> bool: ...
def is_valid_short_number(numobj: PhoneNumber) -> bool: ...
def expected_cost_for_region(short_numobj: PhoneNumber, region_dialing_from: str) -> int: ...
def expected_cost(numobj: PhoneNumber) -> int: ...
def _region_code_for_short_number_from_region_list(numobj: PhoneNumber, region_codes: tuple[str, ...]) -> str | None: ...
def _example_short_number(region_code: str) -> str: ...
def _example_short_number_for_cost(region_code: str, cost: int) -> str: ...
def connects_to_emergency_number(number: str, region_code: str) -> bool: ...
def is_emergency_number(number: str, region_code: str) -> bool: ...
def _matches_emergency_number_helper(number: str, region_code: str, allow_prefix_match: bool) -> bool: ...
def is_carrier_specific(numobj: PhoneNumber) -> bool: ...
def is_carrier_specific_for_region(numobj: PhoneNumber, region_dialing_from: str) -> bool: ...
def is_sms_service_for_region(numobj: PhoneNumber, region_dialing_from: str) -> bool: ...
def _matches_possible_number_and_national_number(number: str, number_desc: PhoneNumberDesc | None) -> bool: ...



from re import Pattern
from re import RegexFlag

from .util import U_EMPTY_STRING, U_SPACE, U_DASH, U_TILDE, U_ZERO, U_SEMICOLON
from .phonemetadata import REGION_CODE_FOR_NON_GEO_ENTITY
from .phonemetadata import NumberFormat
from .phonemetadata import PhoneMetadata
from .phonemetadata import PhoneNumberDesc
from .phonenumber import PhoneNumber
from .util import UnicodeMixin

COUNTRY_CODE_TO_REGION_CODE: dict[int, tuple[str, ...]]
_REGEX_FLAGS: RegexFlag
_MIN_LENGTH_FOR_NSN: int
_MAX_LENGTH_FOR_NSN: int
_MAX_LENGTH_COUNTRY_CODE: int
_MAX_INPUT_STRING_LENGTH: int
UNKNOWN_REGION: str
_NANPA_COUNTRY_CODE: int
_COLOMBIA_MOBILE_TO_FIXED_LINE_PREFIX: str
_MOBILE_TOKEN_MAPPINGS: dict[int, str]
_GEO_MOBILE_COUNTRIES_WITHOUT_MOBILE_AREA_CODES: frozenset[int]
_GEO_MOBILE_COUNTRIES: frozenset[int]
_PLUS_SIGN: str
_STAR_SIGN: str
_RFC3966_EXTN_PREFIX: str
_RFC3966_PREFIX: str
_RFC3966_PHONE_CONTEXT: str
_RFC3966_ISDN_SUBADDRESS: str
_ASCII_DIGITS_MAP: dict[str, str]
_ALPHA_MAPPINGS: dict[str, str]
_ALPHA_PHONE_MAPPINGS: dict[str, str]
_DIALLABLE_CHAR_MAPPINGS: dict[str, str]
_ALL_PLUS_NUMBER_GROUPING_SYMBOLS: dict[str, str]
_SINGLE_INTERNATIONAL_PREFIX: Pattern[str]
_VALID_PUNCTUATION: str
_DIGITS: str
_VALID_ALPHA: str
_PLUS_CHARS: str
_PLUS_CHARS_PATTERN: Pattern[str]
_SEPARATOR_PATTERN: Pattern[str]
_CAPTURING_DIGIT_PATTERN: Pattern[str]
_VALID_START_CHAR: str
_VALID_START_CHAR_PATTERN: Pattern[str]
_SECOND_NUMBER_START: str
_SECOND_NUMBER_START_PATTERN: Pattern[str]
_UNWANTED_END_CHARS: str
_UNWANTED_END_CHAR_PATTERN: Pattern[str]
_VALID_ALPHA_PHONE_PATTERN: Pattern[str]
_VALID_PHONE_NUMBER: str
_DEFAULT_EXTN_PREFIX: str

def _extn_digits(max_length: int) -> str: ...
def _create_extn_pattern(for_parsing: bool) -> str: ...

_EXTN_PATTERNS_FOR_PARSING: str
_EXTN_PATTERNS_FOR_MATCHING: str
_RFC3966_VISUAL_SEPARATOR: str
_RFC3966_PHONE_DIGIT: str
_RFC3966_GLOBAL_NUMBER_DIGITS: str
_RFC3966_GLOBAL_NUMBER_DIGITS_PATTERN: Pattern[str]
_ALPHANUM: str
_RFC3966_DOMAINLABEL: str
_RFC3966_TOPLABEL: str
_RFC3966_DOMAINNAME: str
_RFC3966_DOMAINNAME_PATTERN: Pattern[str]
_EXTN_PATTERN: Pattern[str]
_VALID_PHONE_NUMBER_PATTERN: Pattern[str]
NON_DIGITS_PATTERN: Pattern[str]
_FIRST_GROUP_PATTERN: Pattern[str]
_NP_STRING: str
_FG_STRING: str
_CC_STRING: str
_FIRST_GROUP_ONLY_PREFIX_PATTERN: Pattern[str]

class PhoneNumberFormat:
    E164: int
    INTERNATIONAL: int
    NATIONAL: int
    RFC3966: int
    @classmethod
    def to_string(cls, val: int) -> str: ...

class PhoneNumberType:
    FIXED_LINE: int
    MOBILE: int
    FIXED_LINE_OR_MOBILE: int
    TOLL_FREE: int
    PREMIUM_RATE: int
    SHARED_COST: int
    VOIP: int
    PERSONAL_NUMBER: int
    PAGER: int
    UAN: int
    VOICEMAIL: int
    UNKNOWN: int
    @classmethod
    def values(cls) -> tuple[int, ...]: ...
    @classmethod
    def to_string(cls, val: int) -> str: ...

class MatchType:
    NOT_A_NUMBER: int
    NO_MATCH: int
    SHORT_NSN_MATCH: int
    NSN_MATCH: int
    EXACT_MATCH: int
    @classmethod
    def to_string(cls, val: int) -> str: ...

class ValidationResult:
    IS_POSSIBLE: int
    IS_POSSIBLE_LOCAL_ONLY: int
    INVALID_COUNTRY_CODE: int
    TOO_SHORT: int
    INVALID_LENGTH: int
    TOO_LONG: int
    @classmethod
    def to_string(cls, val: int) -> str: ...

SUPPORTED_REGIONS: set[str]
COUNTRY_CODES_FOR_NON_GEO_REGIONS: set[int]
_NANPA_REGIONS: set[str]

def _regenerate_derived_data() -> None: ...
def _copy_number_format(other: NumberFormat) -> NumberFormat: ...
def _extract_possible_number(number: str) -> str: ...
def _is_viable_phone_number(number: str) -> bool: ...
def _normalize(number: str) -> str: ...
def normalize_digits_only(number: str, keep_non_digits: bool = ...) -> str: ...
def normalize_diallable_chars_only(number: str) -> str: ...
def convert_alpha_characters_in_number(number: str) -> str: ...
def length_of_geographical_area_code(numobj: PhoneNumber) -> int: ...
def length_of_national_destination_code(numobj: PhoneNumber) -> int: ...
def country_mobile_token(country_code: int) -> str: ...
def _normalize_helper(number: str, replacements: dict[str, str], remove_non_matches: bool) -> str: ...
def supported_calling_codes() -> set[int]: ...
def _desc_has_possible_number_data(desc: PhoneNumberDesc | None) -> bool: ...
def _desc_has_data(desc: PhoneNumberDesc | None) -> bool: ...
def _supported_types_for_metadata(metadata: PhoneMetadata) -> set[int]: ...
def supported_types_for_region(region_code: str) -> set[int]: ...
def supported_types_for_non_geo_entity(country_code: int) -> set[int]: ...
def _formatting_rule_has_first_group_only(national_prefix_formatting_rule: str | None) -> bool: ...
def is_number_geographical(numobj: PhoneNumber) -> bool: ...
def is_number_type_geographical(num_type: int, country_code: int) -> bool: ...
def _is_valid_region_code(region_code: str | None) -> bool: ...
def _has_valid_country_calling_code(country_calling_code: int) -> bool: ...
def format_number(numobj: PhoneNumber, num_format: int) -> str: ...
def format_by_pattern(numobj: PhoneNumber, number_format: int, user_defined_formats: list[NumberFormat]) -> str: ...
def format_national_number_with_carrier_code(numobj: PhoneNumber, carrier_code: str) -> str: ...
def format_national_number_with_preferred_carrier_code(numobj: PhoneNumber, fallback_carrier_code: str) -> str: ...
def format_number_for_mobile_dialing(numobj: PhoneNumber, region_calling_from: str, with_formatting: bool) -> str: ...
def format_out_of_country_calling_number(numobj: PhoneNumber, region_calling_from: str) -> str: ...
def format_in_original_format(numobj: PhoneNumber, region_calling_from: str) -> str: ...
def _format_original_allow_mods(numobj: PhoneNumber, region_calling_from: str, format_rule: NumberFormat) -> str: ...
def _raw_input_contains_national_prefix(raw_input: str, national_prefix: str, region_code: str) -> bool: ...
def _choose_formatting_pattern_for_numobj(numobj: PhoneNumber) -> NumberFormat: ...
def _choose_formatting_pattern_for_number(available_formats: list[NumberFormat], national_number: str) -> NumberFormat | None: ...
def format_out_of_country_keeping_alpha_chars(numobj: PhoneNumber, region_calling_from: str) -> str: ...
def national_significant_number(numobj: PhoneNumber) -> str: ...
def _prefix_number_with_country_calling_code(country_code: int, num_format: int, formatted_number: str) -> str: ...
def _format_nsn(number: str, metadata: PhoneMetadata, num_format: int, carrier_code: str | None = ...) -> str: ...
def _format_nsn_using_pattern(national_number: str, formatting_pattern: NumberFormat, number_format: int, carrier_code: str | None = ...) -> str: ...
def example_number(region_code: str) -> PhoneNumber | None: ...
def invalid_example_number(region_code: str) -> PhoneNumber | None: ...
def example_number_for_type(region_code: str | None, num_type: int) -> PhoneNumber | None: ...
def _example_number_anywhere_for_type(num_type: int) -> PhoneNumber | None: ...
def example_number_for_non_geo_entity(country_calling_code: int) -> PhoneNumber | None: ...
def _maybe_append_formatted_extension(numobj: PhoneNumber, metadata: PhoneMetadata, num_format: int, number: str) -> str: ...
def _number_desc_by_type(metadata: PhoneMetadata, num_type: int) -> PhoneNumberDesc | None: ...
def number_type(numobj: PhoneNumber) -> int: ...
def _number_type_helper(national_number: str, metadata: PhoneMetadata) -> int: ...
def _is_number_matching_desc(national_number: str, number_desc: PhoneNumberDesc | None) -> bool: ...
def is_valid_number(numobj: PhoneNumber) -> bool: ...
def is_valid_number_for_region(numobj: PhoneNumber, region_code: str) -> bool: ...
def region_code_for_number(numobj: PhoneNumber) -> str | None: ...
def _region_code_for_number_from_list(numobj: PhoneNumber, regions: tuple[str, ...]) -> str | None: ...
def region_code_for_country_code(country_code: int) -> str: ...
def region_codes_for_country_code(country_code: int) -> tuple[str, ...]: ...
def country_code_for_region(region_code: str) -> int: ...
def country_code_for_valid_region(region_code: str) -> int: ...
def ndd_prefix_for_region(region_code: str, strip_non_digits: bool) -> str | None: ...
def is_nanpa_country(region_code: str) -> bool: ...
def is_alpha_number(number: str) -> bool: ...
def is_possible_number(numobj: PhoneNumber) -> bool: ...
def is_possible_number_for_type(numobj: PhoneNumber, numtype: int) -> bool: ...
def _test_number_length(national_number: str, metadata: PhoneMetadata, numtype: int = ...) -> int: ...
def is_possible_number_with_reason(numobj: PhoneNumber) -> int: ...
def is_possible_number_for_type_with_reason(numobj: PhoneNumber, numtype: int) -> int: ...
def is_possible_number_string(number: str, region_dialing_from: str) -> bool: ...
def truncate_too_long_number(numobj: PhoneNumber) -> bool: ...
def _extract_country_code(number: str) -> tuple[int, str]: ...
def _maybe_extract_country_code(number: str, metadata: PhoneMetadata, keep_raw_input: bool, numobj: PhoneNumber) -> tuple[int, str]: ...
def _parse_prefix_as_idd(idd_pattern: Pattern[str], number: str) -> tuple[bool, str]: ...
def _maybe_strip_i18n_prefix_and_normalize(number: str, possible_idd_prefix: str) -> tuple[int, str]: ...
def _maybe_strip_national_prefix_carrier_code(number: str, metadata: PhoneMetadata) -> tuple[str, str, bool]: ...
def _maybe_strip_extension(number: str) -> tuple[str, str]: ...
def _check_region_for_parsing(number: str | None, default_region: str | None) -> bool: ...
def _set_italian_leading_zeros_for_phone_number(national_number: str, numobj: PhoneNumber) -> None: ...
def parse(number: str, region: str | None = ..., keep_raw_input: bool = ..., numobj: PhoneNumber | None = ..., _check_region: bool = ...) -> PhoneNumber: ...
def _extract_phone_context(number_to_extract_from: str, index_of_phone_context:int) -> str | None: ...
def _is_phone_context_valid(phone_context: str | None) -> bool: ...
def _build_national_number_for_parsing(number: str) -> str: ...
def _copy_core_fields_only(inobj: PhoneNumber) -> PhoneNumber: ...
def _is_number_match_OO(numobj1_in: PhoneNumber, numobj2_in: PhoneNumber) -> int: ...
def _is_national_number_suffix_of_other(numobj1: PhoneNumber, numobj2: PhoneNumber) -> bool: ...
def _is_number_match_SS(number1: str, number2: str) -> int: ...
def _is_number_match_OS(numobj1: PhoneNumber, number2: str) -> int: ...
def is_number_match(num1: PhoneNumber | str, num2: PhoneNumber | str) -> int: ...
def can_be_internationally_dialled(numobj: PhoneNumber) -> bool: ...
def is_mobile_number_portable_region(region_code: str) -> bool: ...

class NumberParseException(UnicodeMixin, Exception):
    INVALID_COUNTRY_CODE: int
    NOT_A_NUMBER: int
    TOO_SHORT_AFTER_IDD: int
    TOO_SHORT_NSN: int
    TOO_LONG: int
    error_type: int
    _msg: str
    def __init__(self, error_type: int, msg: str) -> None: ...
    def __reduce__(self) -> tuple[type[NumberParseException], tuple[int, str]]: ...
    def __unicode__(self) -> str: ...

def _match_national_number(number: str, number_desc: PhoneNumberDesc | None, allow_prefix_match: bool) -> bool: ...
def _match(number: str, pattern: Pattern[str], allow_prefix_match: bool) -> bool: ...



"""Auto-generated file, do not edit by hand. BJ metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_BJ = PhoneMetadata(id='BJ', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[17]\\d{2,3}', possible_length=(3, 4)),
    toll_free=PhoneNumberDesc(national_number_pattern='1(?:1[246-8]|3[68]|6[06])|7[3-5]\\d\\d', example_number='112', possible_length=(3, 4)),
    emergency=PhoneNumberDesc(national_number_pattern='11[246-8]', example_number='112', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:05|1[24-8]|2[02-5]|3[126-8]|5[05]|6[06]|89)|7[0-5]\\d\\d', example_number='105', possible_length=(3, 4)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='12[02-5]', example_number='120', possible_length=(3,)),
    short_data=True)



"""Auto-generated file, do not edit by hand. MY metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_MY = PhoneMetadata(id='MY', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[1369]\\d{2,4}', possible_length=(3, 4, 5)),
    toll_free=PhoneNumberDesc(national_number_pattern='112|999', example_number='112', possible_length=(3,)),
    emergency=PhoneNumberDesc(national_number_pattern='112|999', example_number='112', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:0[01348]|1(?:[02]|1[128]|311)|2(?:0[125]|[13-6]|2\\d{0,2})|(?:3[1-35-79]|7[45])\\d\\d?|5(?:454|5\\d\\d?|77|888|999?)|8(?:18?|2|8[18])|9(?:[124]\\d?|68|71|9[0679]))|66628|99[1-469]|13[5-7]|(?:1(?:0[569]|309|5[12]|7[136-9]|9[03])|3[23679]\\d\\d)\\d', example_number='100', possible_length=(3, 4, 5)),
    standard_rate=PhoneNumberDesc(national_number_pattern='666\\d\\d', example_number='66600', possible_length=(5,)),
    sms_services=PhoneNumberDesc(national_number_pattern='(?:3[23679]\\d|666)\\d\\d', example_number='32000', possible_length=(5,)),
    short_data=True)



from .util import UnicodeMixin

class Category:
    LETTER: str
    UPPERCASE_LETTER: str
    LOWERCASE_LETTER: str
    TITLECASE_LETTER: str
    MODIFIER_LETTER: str
    OTHER_LETTER: str
    MARK: str
    NON_SPACING_MARK: str
    SPACING_COMBINING_MARK: str
    ENCLOSING_MARK: str
    NUMBER: str
    DECIMAL_DIGIT_NUMBER: str
    LETTER_NUMBER: str
    OTHER_NUMBER: str
    SYMBOL: str
    MATH_SYMBOL: str
    CURRENCY_SYMBOL: str
    MODIFIER_SYMBOL: str
    OTHER_SYMBOL: str
    PUNCTUATION: str
    CONNECTOR_PUNCTUATION: str
    DASH_PUNCTUATION: str
    OPEN_PUNCTUATION: str
    CLOSE_PUNCTUATION: str
    INITIAL_PUNCTUATION: str
    FINAL_PUNCTUATION: str
    OTHER_PUNCTUATION: str
    SEPARATOR: str
    SPACE_SEPARATOR: str
    LINE_SEPARATOR: str
    PARAGRAPH_SEPARATOR: str
    OTHER: str
    CONTROL: str
    FORMAT: str
    SURROGATE: str
    PRIVATE_USE: str
    NOT_ASSIGNED: str
    @classmethod
    def get(cls, uni_char: str) -> str: ...

def is_letter(uni_char: str) -> bool: ...

class _BlockRange(UnicodeMixin):
    start: int
    end: int
    def __init__(self, start: int, end: int, regdict: dict[int, _BlockRange] | None = ...) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __unicode__(self) -> str: ...

class Block:
    _RANGES: dict[int, _BlockRange]
    _RANGE_KEYS: list[int] | None
    BASIC_LATIN: _BlockRange
    LATIN_1_SUPPLEMENT: _BlockRange
    LATIN_EXTENDED_A: _BlockRange
    LATIN_EXTENDED_B: _BlockRange
    IPA_EXTENSIONS: _BlockRange
    SPACING_MODIFIER_LETTERS: _BlockRange
    COMBINING_DIACRITICAL_MARKS: _BlockRange
    GREEK_AND_COPTIC: _BlockRange
    CYRILLIC: _BlockRange
    CYRILLIC_SUPPLEMENT: _BlockRange
    ARMENIAN: _BlockRange
    HEBREW: _BlockRange
    ARABIC: _BlockRange
    SYRIAC: _BlockRange
    ARABIC_SUPPLEMENT: _BlockRange
    THAANA: _BlockRange
    NKO: _BlockRange
    SAMARITAN: _BlockRange
    MANDAIC: _BlockRange
    DEVANAGARI: _BlockRange
    BENGALI: _BlockRange
    GURMUKHI: _BlockRange
    GUJARATI: _BlockRange
    ORIYA: _BlockRange
    TAMIL: _BlockRange
    TELUGU: _BlockRange
    KANNADA: _BlockRange
    MALAYALAM: _BlockRange
    SINHALA: _BlockRange
    THAI: _BlockRange
    LAO: _BlockRange
    TIBETAN: _BlockRange
    MYANMAR: _BlockRange
    GEORGIAN: _BlockRange
    HANGUL_JAMO: _BlockRange
    ETHIOPIC: _BlockRange
    ETHIOPIC_SUPPLEMENT: _BlockRange
    CHEROKEE: _BlockRange
    UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS: _BlockRange
    OGHAM: _BlockRange
    RUNIC: _BlockRange
    TAGALOG: _BlockRange
    HANUNOO: _BlockRange
    BUHID: _BlockRange
    TAGBANWA: _BlockRange
    KHMER: _BlockRange
    MONGOLIAN: _BlockRange
    UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS_EXTENDED: _BlockRange
    LIMBU: _BlockRange
    TAI_LE: _BlockRange
    NEW_TAI_LUE: _BlockRange
    KHMER_SYMBOLS: _BlockRange
    BUGINESE: _BlockRange
    TAI_THAM: _BlockRange
    BALINESE: _BlockRange
    SUNDANESE: _BlockRange
    BATAK: _BlockRange
    LEPCHA: _BlockRange
    OL_CHIKI: _BlockRange
    VEDIC_EXTENSIONS: _BlockRange
    PHONETIC_EXTENSIONS: _BlockRange
    PHONETIC_EXTENSIONS_SUPPLEMENT: _BlockRange
    COMBINING_DIACRITICAL_MARKS_SUPPLEMENT: _BlockRange
    LATIN_EXTENDED_ADDITIONAL: _BlockRange
    GREEK_EXTENDED: _BlockRange
    GENERAL_PUNCTUATION: _BlockRange
    SUPERSCRIPTS_AND_SUBSCRIPTS: _BlockRange
    CURRENCY_SYMBOLS: _BlockRange
    COMBINING_DIACRITICAL_MARKS_FOR_SYMBOLS: _BlockRange
    LETTERLIKE_SYMBOLS: _BlockRange
    NUMBER_FORMS: _BlockRange
    ARROWS: _BlockRange
    MATHEMATICAL_OPERATORS: _BlockRange
    MISCELLANEOUS_TECHNICAL: _BlockRange
    CONTROL_PICTURES: _BlockRange
    OPTICAL_CHARACTER_RECOGNITION: _BlockRange
    ENCLOSED_ALPHANUMERICS: _BlockRange
    BOX_DRAWING: _BlockRange
    BLOCK_ELEMENTS: _BlockRange
    GEOMETRIC_SHAPES: _BlockRange
    MISCELLANEOUS_SYMBOLS: _BlockRange
    DINGBATS: _BlockRange
    MISCELLANEOUS_MATHEMATICAL_SYMBOLS_A: _BlockRange
    SUPPLEMENTAL_ARROWS_A: _BlockRange
    BRAILLE_PATTERNS: _BlockRange
    SUPPLEMENTAL_ARROWS_B: _BlockRange
    MISCELLANEOUS_MATHEMATICAL_SYMBOLS_B: _BlockRange
    SUPPLEMENTAL_MATHEMATICAL_OPERATORS: _BlockRange
    MISCELLANEOUS_SYMBOLS_AND_ARROWS: _BlockRange
    GLAGOLITIC: _BlockRange
    LATIN_EXTENDED_C: _BlockRange
    COPTIC: _BlockRange
    GEORGIAN_SUPPLEMENT: _BlockRange
    TIFINAGH: _BlockRange
    ETHIOPIC_EXTENDED: _BlockRange
    CYRILLIC_EXTENDED_A: _BlockRange
    SUPPLEMENTAL_PUNCTUATION: _BlockRange
    CJK_RADICALS_SUPPLEMENT: _BlockRange
    KANGXI_RADICALS: _BlockRange
    IDEOGRAPHIC_DESCRIPTION_CHARACTERS: _BlockRange
    CJK_SYMBOLS_AND_PUNCTUATION: _BlockRange
    HIRAGANA: _BlockRange
    KATAKANA: _BlockRange
    BOPOMOFO: _BlockRange
    HANGUL_COMPATIBILITY_JAMO: _BlockRange
    KANBUN: _BlockRange
    BOPOMOFO_EXTENDED: _BlockRange
    CJK_STROKES: _BlockRange
    KATAKANA_PHONETIC_EXTENSIONS: _BlockRange
    ENCLOSED_CJK_LETTERS_AND_MONTHS: _BlockRange
    CJK_COMPATIBILITY: _BlockRange
    CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A: _BlockRange
    YIJING_HEXAGRAM_SYMBOLS: _BlockRange
    CJK_UNIFIED_IDEOGRAPHS: _BlockRange
    YI_SYLLABLES: _BlockRange
    YI_RADICALS: _BlockRange
    LISU: _BlockRange
    VAI: _BlockRange
    CYRILLIC_EXTENDED_B: _BlockRange
    BAMUM: _BlockRange
    MODIFIER_TONE_LETTERS: _BlockRange
    LATIN_EXTENDED_D: _BlockRange
    SYLOTI_NAGRI: _BlockRange
    COMMON_INDIC_NUMBER_FORMS: _BlockRange
    PHAGS_PA: _BlockRange
    SAURASHTRA: _BlockRange
    DEVANAGARI_EXTENDED: _BlockRange
    KAYAH_LI: _BlockRange
    REJANG: _BlockRange
    HANGUL_JAMO_EXTENDED_A: _BlockRange
    JAVANESE: _BlockRange
    CHAM: _BlockRange
    MYANMAR_EXTENDED_A: _BlockRange
    TAI_VIET: _BlockRange
    ETHIOPIC_EXTENDED_A: _BlockRange
    MEETEI_MAYEK: _BlockRange
    HANGUL_SYLLABLES: _BlockRange
    HANGUL_JAMO_EXTENDED_B: _BlockRange
    HIGH_SURROGATES: _BlockRange
    HIGH_PRIVATE_USE_SURROGATES: _BlockRange
    LOW_SURROGATES: _BlockRange
    PRIVATE_USE_AREA: _BlockRange
    CJK_COMPATIBILITY_IDEOGRAPHS: _BlockRange
    ALPHABETIC_PRESENTATION_FORMS: _BlockRange
    ARABIC_PRESENTATION_FORMS_A: _BlockRange
    VARIATION_SELECTORS: _BlockRange
    VERTICAL_FORMS: _BlockRange
    COMBINING_HALF_MARKS: _BlockRange
    CJK_COMPATIBILITY_FORMS: _BlockRange
    SMALL_FORM_VARIANTS: _BlockRange
    ARABIC_PRESENTATION_FORMS_B: _BlockRange
    HALFWIDTH_AND_FULLWIDTH_FORMS: _BlockRange
    SPECIALS: _BlockRange
    LINEAR_B_SYLLABARY: _BlockRange
    LINEAR_B_IDEOGRAMS: _BlockRange
    AEGEAN_NUMBERS: _BlockRange
    ANCIENT_GREEK_NUMBERS: _BlockRange
    ANCIENT_SYMBOLS: _BlockRange
    PHAISTOS_DISC: _BlockRange
    LYCIAN: _BlockRange
    CARIAN: _BlockRange
    OLD_ITALIC: _BlockRange
    GOTHIC: _BlockRange
    UGARITIC: _BlockRange
    OLD_PERSIAN: _BlockRange
    DESERET: _BlockRange
    SHAVIAN: _BlockRange
    OSMANYA: _BlockRange
    CYPRIOT_SYLLABARY: _BlockRange
    IMPERIAL_ARAMAIC: _BlockRange
    PHOENICIAN: _BlockRange
    LYDIAN: _BlockRange
    KHAROSHTHI: _BlockRange
    OLD_SOUTH_ARABIAN: _BlockRange
    AVESTAN: _BlockRange
    INSCRIPTIONAL_PARTHIAN: _BlockRange
    INSCRIPTIONAL_PAHLAVI: _BlockRange
    OLD_TURKIC: _BlockRange
    RUMI_NUMERAL_SYMBOLS: _BlockRange
    BRAHMI: _BlockRange
    KAITHI: _BlockRange
    CUNEIFORM: _BlockRange
    CUNEIFORM_NUMBERS_AND_PUNCTUATION: _BlockRange
    EGYPTIAN_HIEROGLYPHS: _BlockRange
    BAMUM_SUPPLEMENT: _BlockRange
    KANA_SUPPLEMENT: _BlockRange
    BYZANTINE_MUSICAL_SYMBOLS: _BlockRange
    MUSICAL_SYMBOLS: _BlockRange
    ANCIENT_GREEK_MUSICAL_NOTATION: _BlockRange
    TAI_XUAN_JING_SYMBOLS: _BlockRange
    COUNTING_ROD_NUMERALS: _BlockRange
    MATHEMATICAL_ALPHANUMERIC_SYMBOLS: _BlockRange
    MAHJONG_TILES: _BlockRange
    DOMINO_TILES: _BlockRange
    PLAYING_CARDS: _BlockRange
    ENCLOSED_ALPHANUMERIC_SUPPLEMENT: _BlockRange
    ENCLOSED_IDEOGRAPHIC_SUPPLEMENT: _BlockRange
    MISCELLANEOUS_SYMBOLS_AND_PICTOGRAPHS: _BlockRange
    EMOTICONS: _BlockRange
    TRANSPORT_AND_MAP_SYMBOLS: _BlockRange
    ALCHEMICAL_SYMBOLS: _BlockRange
    CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B: _BlockRange
    CJK_UNIFIED_IDEOGRAPHS_EXTENSION_C: _BlockRange
    CJK_UNIFIED_IDEOGRAPHS_EXTENSION_D: _BlockRange
    CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT: _BlockRange
    TAGS: _BlockRange
    VARIATION_SELECTORS_SUPPLEMENT: _BlockRange
    SUPPLEMENTARY_PRIVATE_USE_AREA_A: _BlockRange
    SUPPLEMENTARY_PRIVATE_USE_AREA_B: _BlockRange
    UNKNOWN: _BlockRange
    @classmethod
    def get(cls, uni_char: str) -> _BlockRange: ...

def digit(uni_char: str, default_value: int | None = ...) -> int: ...



"""Auto-generated file, do not edit by hand. GI metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_GI = PhoneMetadata(id='GI', country_code=350, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:[25]\\d|60)\\d{6}', possible_length=(8,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='2190[0-2]\\d{3}|2(?:0(?:[02]\\d|3[01])|16[24-9]|2[2-5]\\d)\\d{4}', example_number='20012345', possible_length=(8,)),
    mobile=PhoneNumberDesc(national_number_pattern='5251[0-4]\\d{3}|(?:5(?:[146-8]\\d\\d|250)|60(?:1[01]|6\\d))\\d{4}', example_number='57123456', possible_length=(8,)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{5})', format='\\1 \\2', leading_digits_pattern=['2'])],
    mobile_number_portable_region=True)



from collections.abc import Callable
import threading

from .util import ImmutableMixin
from .util import UnicodeMixin

REGION_CODE_FOR_NON_GEO_ENTITY: str

class NumberFormat(UnicodeMixin, ImmutableMixin):
    pattern: str | None
    format: str | None
    leading_digits_pattern: list[str]
    national_prefix_formatting_rule: str | None
    national_prefix_optional_when_formatting: bool | None
    domestic_carrier_code_formatting_rule: str | None
    def __init__(
        self,
        pattern: str | None = ...,
        format: str | None = ...,
        leading_digits_pattern: list[str] | None = ...,
        national_prefix_formatting_rule: str | None = ...,
        national_prefix_optional_when_formatting: bool | None = ...,
        domestic_carrier_code_formatting_rule: str | None = ...,
    ) -> None: ...
    def merge_from(self, other: NumberFormat) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __repr__(self) -> str: ...
    def __unicode__(self) -> str: ...

class PhoneNumberDesc(UnicodeMixin, ImmutableMixin):
    national_number_pattern: str | None
    example_number: str | None
    possible_length: tuple[int, ...]
    possible_length_local_only: tuple[int, ...]
    def __init__(
        self,
        national_number_pattern: str | None = ...,
        example_number: str | None = ...,
        possible_length: tuple[int, ...] | None = ...,
        possible_length_local_only: tuple[int, ...] | None = ...,
    ) -> None: ...
    def merge_from(self, other: PhoneNumberDesc) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __repr__(self) -> str: ...
    def __unicode__(self) -> str: ...

def _same_pattern(left: PhoneNumberDesc | None, right: PhoneNumberDesc | None) -> bool: ...

class PhoneMetadata(UnicodeMixin, ImmutableMixin):
    _metadata_lock: threading.Lock
    _region_available: dict[str, Callable[[str], None] | None]
    _short_region_available: dict[str, Callable[[str], None] | None]
    _country_code_available: dict[int, Callable[[int], None] | None]
    _region_metadata: dict[str, PhoneMetadata]
    _short_region_metadata: dict[str, PhoneMetadata]
    _country_code_metadata: dict[int, PhoneMetadata]
    general_desc: PhoneNumberDesc | None
    fixed_line: PhoneNumberDesc | None
    mobile: PhoneNumberDesc | None
    toll_free: PhoneNumberDesc | None
    premium_rate: PhoneNumberDesc | None
    shared_cost: PhoneNumberDesc | None
    personal_number: PhoneNumberDesc | None
    voip: PhoneNumberDesc | None
    pager: PhoneNumberDesc | None
    uan: PhoneNumberDesc | None
    emergency: PhoneNumberDesc | None
    voicemail: PhoneNumberDesc | None
    short_code: PhoneNumberDesc | None
    standard_rate: PhoneNumberDesc | None
    carrier_specific: PhoneNumberDesc | None
    sms_services: PhoneNumberDesc | None
    no_international_dialling: PhoneNumberDesc | None
    id: str
    country_code: int | None
    international_prefix: str | None
    preferred_international_prefix: str | None
    national_prefix: str | None
    preferred_extn_prefix: str | None
    national_prefix_for_parsing: str | None
    national_prefix_transform_rule: str | None
    same_mobile_and_fixed_line_pattern: bool
    number_format: list[NumberFormat]
    intl_number_format: list[NumberFormat]
    main_country_for_code: bool
    leading_digits: str | None
    leading_zero_possible: bool
    mobile_number_portable_region: bool
    short_data: bool
    @classmethod
    def metadata_for_region(cls, region_code: str, default: PhoneMetadata | None = ...) -> PhoneMetadata | None: ...
    @classmethod
    def short_metadata_for_region(cls, region_code: str, default: PhoneMetadata | None = ...) -> PhoneMetadata | None: ...
    @classmethod
    def metadata_for_nongeo_region(cls, country_code: int, default: PhoneMetadata | None = ...) -> PhoneMetadata | None: ...
    @classmethod
    def metadata_for_region_or_calling_code(cls, country_calling_code: int, region_code: str) -> PhoneMetadata | None: ...
    @classmethod
    def register_region_loader(cls, region_code: str, loader: Callable[[str], None]) -> None: ...
    @classmethod
    def register_short_region_loader(cls, region_code: str, loader: Callable[[str], None]) -> None: ...
    @classmethod
    def register_nongeo_region_loader(cls, country_code: int, loader: Callable[[int], None]) -> None: ...
    @classmethod
    def load_all(cls) -> None: ...
    def __init__(
        self,
        id: str,
        general_desc: PhoneNumberDesc | None = ...,
        fixed_line: PhoneNumberDesc | None = ...,
        mobile: PhoneNumberDesc | None = ...,
        toll_free: PhoneNumberDesc | None = ...,
        premium_rate: PhoneNumberDesc | None = ...,
        shared_cost: PhoneNumberDesc | None = ...,
        personal_number: PhoneNumberDesc | None = ...,
        voip: PhoneNumberDesc | None = ...,
        pager: PhoneNumberDesc | None = ...,
        uan: PhoneNumberDesc | None = ...,
        emergency: PhoneNumberDesc | None = ...,
        voicemail: PhoneNumberDesc | None = ...,
        short_code: PhoneNumberDesc | None = ...,
        standard_rate: PhoneNumberDesc | None = ...,
        carrier_specific: PhoneNumberDesc | None = ...,
        sms_services: PhoneNumberDesc | None = ...,
        no_international_dialling: PhoneNumberDesc | None = ...,
        country_code: int | None = ...,
        international_prefix: str | None = ...,
        preferred_international_prefix: str | None = ...,
        national_prefix: str | None = ...,
        preferred_extn_prefix: str | None = ...,
        national_prefix_for_parsing: str | None = ...,
        national_prefix_transform_rule: str | None = ...,
        number_format: list[NumberFormat] | None = ...,
        intl_number_format: list[NumberFormat] | None = ...,
        main_country_for_code: bool = ...,
        leading_digits: str | None = ...,
        leading_zero_possible: bool = ...,
        mobile_number_portable_region: bool = ...,
        short_data: bool = ...,
        register: bool = ...,
    ) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __repr__(self) -> str: ...
    def __unicode__(self) -> str: ...



"""Auto-generated file, do not edit by hand. XK metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_XK = PhoneMetadata(id='XK', country_code=383, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='2\\d{7,8}|3\\d{7,11}|(?:4\\d\\d|[89]00)\\d{5}', possible_length=(8, 9, 10, 11, 12)),
    fixed_line=PhoneNumberDesc(national_number_pattern='38\\d{6,10}|(?:2[89]|39)(?:0\\d{5,6}|[1-9]\\d{5})', example_number='28012345', possible_length=(8, 9, 10, 11, 12)),
    mobile=PhoneNumberDesc(national_number_pattern='4[3-9]\\d{6}', example_number='43201234', possible_length=(8,)),
    toll_free=PhoneNumberDesc(national_number_pattern='800\\d{5}', example_number='80001234', possible_length=(8,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='900\\d{5}', example_number='90001234', possible_length=(8,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{5})', format='\\1 \\2', leading_digits_pattern=['[89]'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{2})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['[2-4]'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['2|39'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{2})(\\d{7,10})', format='\\1 \\2', leading_digits_pattern=['3'], national_prefix_formatting_rule='0\\1')])



"""Auto-generated file, do not edit by hand. BD metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_BD = PhoneMetadata(id='BD', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[1579]\\d{2,4}', possible_length=(3, 4, 5)),
    toll_free=PhoneNumberDesc(national_number_pattern='10[0-26]|[19]99', example_number='100', possible_length=(3,)),
    emergency=PhoneNumberDesc(national_number_pattern='10[0-2]|[19]99', example_number='100', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:0(?:[0-369]|5[1-4]|7[0-4]|8[0-29])|1[16-9]|2(?:[134]|2[0-5])|3(?:1\\d?|6[3-6])|5[2-9])|5012|786|9594|[19]99|1(?:0(?:50|6\\d)|33|4(?:0|1\\d))\\d', example_number='100', possible_length=(3, 4, 5)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='1(?:11|2[13])|(?:501|959)\\d|786', example_number='111', possible_length=(3, 4)),
    sms_services=PhoneNumberDesc(national_number_pattern='959\\d', example_number='9590', possible_length=(4,)),
    short_data=True)



from collections.abc import Callable
from typing import Any, overload

from _typeshed import SupportsWrite

unicod: type[str]
u: type[str]
to_long: type[int]

@overload
def prnt(
    *values: object,
    sep: str | None = ...,
    end: str | None = ...,
    file: SupportsWrite[str] | None = ...,
) -> None: ...
@overload
def prnt(*values: object, **kwargs: Any) -> None: ...

class UnicodeMixin:
    def __str__(self) -> str: ...

U_EMPTY_STRING: str
U_SPACE: str
U_DASH: str
U_TILDE: str
U_PLUS: str
U_STAR: str
U_ZERO: str
U_SLASH: str
U_SEMICOLON: str
U_X_LOWER: str
U_X_UPPER: str
U_PERCENT: str

def rpr(s: str | None) -> str: ...
@overload
def force_unicode(s: None) -> None: ...
@overload
def force_unicode(s: str) -> str: ...

class ImmutableMixin:
    _mutable: bool
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...

def mutating_method(func: Callable[..., Any]) -> Callable[..., Any]: ...



"""Auto-generated file, do not edit by hand. SA metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_SA = PhoneMetadata(id='SA', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[19]\\d{2,5}', possible_length=(3, 4, 5, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='11(?:2|6\\d{3})|9(?:11|37|9[7-9])', example_number='112', possible_length=(3, 6)),
    emergency=PhoneNumberDesc(national_number_pattern='112|9(?:11|9[79])', example_number='112', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:1(?:00|2|6111)|410|9(?:00|1[89]|9(?:099|22|9[0-3])))|9(?:0[24-79]|11|3[379]|40|66|8[5-9]|9[02-9])', example_number='112', possible_length=(3, 4, 5, 6)),
    standard_rate=PhoneNumberDesc(national_number_pattern='141\\d', example_number='1410', possible_length=(4,)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='1(?:10|41)\\d|90[24679]', example_number='902', possible_length=(3, 4)),
    short_data=True)



"""Auto-generated file, do not edit by hand. SY metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_SY = PhoneMetadata(id='SY', country_code=963, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[1-359]\\d{8}|[1-5]\\d{7}', possible_length=(8, 9), possible_length_local_only=(6, 7)),
    fixed_line=PhoneNumberDesc(national_number_pattern='21\\d{6,7}|(?:1(?:[14]\\d|[2356])|2[235]|3(?:[13]\\d|4)|4[134]|5[1-3])\\d{6}', example_number='112345678', possible_length=(8, 9), possible_length_local_only=(6, 7)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:50|9[1-9])\\d{7}', example_number='944567890', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{3})(\\d{3,4})', format='\\1 \\2 \\3', leading_digits_pattern=['[1-4]|5[1-3]'], national_prefix_formatting_rule='0\\1', national_prefix_optional_when_formatting=True),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['[59]'], national_prefix_formatting_rule='0\\1', national_prefix_optional_when_formatting=True)])



"""Auto-generated file, do not edit by hand. FR metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_FR = PhoneMetadata(id='FR', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[1-8]\\d{1,5}', possible_length=(2, 3, 4, 5, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='1(?:1[02459]|[578]|9[167])|224|(?:3370|74)0|(?:116\\d|3[01])\\d\\d', example_number='15', possible_length=(2, 3, 4, 5, 6)),
    premium_rate=PhoneNumberDesc(national_number_pattern='(?:1(?:0|18\\d)|366|[4-8]\\d\\d)\\d\\d|3[2-9]\\d\\d', example_number='1000', possible_length=(4, 5, 6)),
    emergency=PhoneNumberDesc(national_number_pattern='1(?:12|[578])', example_number='15', possible_length=(2, 3)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:0\\d\\d|1(?:[02459]|6(?:000|111)|8\\d{3})|[578]|9[167])|2(?:0(?:00|2)0|24)|[3-8]\\d{4}|3\\d{3}|6(?:1[14]|34)|7(?:0[06]|22|40)', example_number='15', possible_length=(2, 3, 4, 5, 6)),
    standard_rate=PhoneNumberDesc(national_number_pattern='202\\d|6(?:1[14]|34)|70[06]', example_number='611', possible_length=(3, 4)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='118777|224|6(?:1[14]|34)|7(?:0[06]|22|40)|20(?:0\\d|2)\\d', example_number='224', possible_length=(3, 4, 5, 6)),
    sms_services=PhoneNumberDesc(national_number_pattern='114|[3-8]\\d{4}', example_number='114', possible_length=(3, 5)),
    short_data=True)



"""Auto-generated file, do not edit by hand. 883 metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_883 = PhoneMetadata(id='001', country_code=883, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='(?:[1-4]\\d|51)\\d{6,10}', possible_length=(8, 9, 10, 11, 12)),
    voip=PhoneNumberDesc(national_number_pattern='(?:2(?:00\\d\\d|10)|(?:370[1-9]|51\\d0)\\d)\\d{7}|51(?:00\\d{5}|[24-9]0\\d{4,7})|(?:1[0-79]|2[24-689]|3[02-689]|4[0-4])0\\d{5,9}', example_number='510012345', possible_length=(8, 9, 10, 11, 12)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{2,8})', format='\\1 \\2 \\3', leading_digits_pattern=['[14]|2[24-689]|3[02-689]|51[24-9]']),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['510']),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['21']),
        NumberFormat(pattern='(\\d{4})(\\d{4})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['51[13]']),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3})(\\d{3})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[235]'])])



"""Auto-generated file, do not edit by hand. HU metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_HU = PhoneMetadata(id='HU', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='1\\d{2,5}', possible_length=(3, 4, 5, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='1(?:0[457]|12|4[0-4]\\d)|1(?:16\\d|37|45)\\d\\d', example_number='104', possible_length=(3, 4, 5, 6)),
    emergency=PhoneNumberDesc(national_number_pattern='1(?:0[457]|12)', example_number='104', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:0[457]|1(?:2|6(?:000|1(?:11|23))|800)|2(?:0[0-4]|1[013489]|2[0-5]|3[0-46]|4[0-24-68]|5[0-2568]|6[06]|7[0-25-7]|8[028]|9[08])|37(?:00|37|7[07])|4(?:0[0-5]|1[013-8]|2[034]|3[23]|4[02-9]|5(?:00|41|67))|777|8(?:1[27-9]|2[04]|40|[589]))', example_number='104', possible_length=(3, 4, 5, 6)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='1(?:4[0-4]|77)\\d|1(?:18|2|45)\\d\\d', example_number='1200', possible_length=(4, 5)),
    sms_services=PhoneNumberDesc(national_number_pattern='184\\d', example_number='1840', possible_length=(4,)),
    short_data=True)



"""Auto-generated file, do not edit by hand. PW metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_PW = PhoneMetadata(id='PW', country_code=680, international_prefix='01[12]',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:[24-8]\\d\\d|345|900)\\d{4}', possible_length=(7,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:2(?:55|77)|345|488|5(?:35|44|87)|6(?:22|54|79)|7(?:33|47)|8(?:24|55|76)|900)\\d{4}', example_number='2771234', possible_length=(7,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:(?:46|83)[0-5]|(?:6[2-4689]|78)0)\\d{4}|(?:45|77|88)\\d{5}', example_number='6201234', possible_length=(7,)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{4})', format='\\1 \\2', leading_digits_pattern=['[2-9]'])])



"""Auto-generated file, do not edit by hand. CI metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_CI = PhoneMetadata(id='CI', country_code=225, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[02]\\d{9}', possible_length=(10,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='2(?:[15]\\d{3}|7(?:2(?:0[23]|1[2357]|2[245]|3[45]|4[3-5])|3(?:06|1[69]|[2-6]7)))\\d{5}', example_number='2123456789', possible_length=(10,)),
    mobile=PhoneNumberDesc(national_number_pattern='0[157]\\d{8}', example_number='0123456789', possible_length=(10,)),
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{2})(\\d)(\\d{5})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['2']),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{4})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['0'])])



"""Auto-generated file, do not edit by hand. OM metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_OM = PhoneMetadata(id='OM', country_code=968, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:1505|[279]\\d{3}|500)\\d{4}|800\\d{5,6}', possible_length=(7, 8, 9)),
    fixed_line=PhoneNumberDesc(national_number_pattern='2[1-6]\\d{6}', example_number='23123456', possible_length=(8,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:1505|90[1-9]\\d)\\d{4}|(?:7[124-9]|9[1-9])\\d{6}', example_number='92123456', possible_length=(8,)),
    toll_free=PhoneNumberDesc(national_number_pattern='8007\\d{4,5}|(?:500|800[05])\\d{4}', example_number='80071234', possible_length=(7, 8, 9)),
    premium_rate=PhoneNumberDesc(national_number_pattern='900\\d{5}', example_number='90012345', possible_length=(8,)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{4,6})', format='\\1 \\2', leading_digits_pattern=['[58]']),
        NumberFormat(pattern='(\\d{2})(\\d{6})', format='\\1 \\2', leading_digits_pattern=['2']),
        NumberFormat(pattern='(\\d{4})(\\d{4})', format='\\1 \\2', leading_digits_pattern=['[179]'])],
    mobile_number_portable_region=True)



"""Auto-generated file, do not edit by hand. LY metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_LY = PhoneMetadata(id='LY', country_code=218, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[2-9]\\d{8}', possible_length=(9,), possible_length_local_only=(7,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:2(?:0[56]|[1-6]\\d|7[124579]|8[124])|3(?:1\\d|2[2356])|4(?:[17]\\d|2[1-357]|5[2-4]|8[124])|5(?:[1347]\\d|2[1-469]|5[13-5]|8[1-4])|6(?:[1-479]\\d|5[2-57]|8[1-5])|7(?:[13]\\d|2[13-79])|8(?:[124]\\d|5[124]|84))\\d{6}', example_number='212345678', possible_length=(9,), possible_length_local_only=(7,)),
    mobile=PhoneNumberDesc(national_number_pattern='9[1-6]\\d{7}', example_number='912345678', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{7})', format='\\1-\\2', leading_digits_pattern=['[2-9]'], national_prefix_formatting_rule='0\\1')])



"""Auto-generated file, do not edit by hand. ET metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_ET = PhoneMetadata(id='ET', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='9\\d\\d?', possible_length=(2, 3)),
    toll_free=PhoneNumberDesc(national_number_pattern='9(?:07|11?|2|39?|9[17])', example_number='91', possible_length=(2, 3)),
    emergency=PhoneNumberDesc(national_number_pattern='9(?:11?|2|39?|9[17])', example_number='91', possible_length=(2, 3)),
    short_code=PhoneNumberDesc(national_number_pattern='9(?:07|11?|2|39?|45|9[17])', example_number='91', possible_length=(2, 3)),
    short_data=True)



"""Auto-generated file, do not edit by hand. MC metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_MC = PhoneMetadata(id='MC', country_code=377, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:[3489]|[67]\\d)\\d{7}', possible_length=(8, 9)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:870|9[2-47-9]\\d)\\d{5}', example_number='99123456', possible_length=(8,)),
    mobile=PhoneNumberDesc(national_number_pattern='4(?:[469]\\d|5[1-9])\\d{5}|(?:3|[67]\\d)\\d{7}', example_number='612345678', possible_length=(8, 9)),
    toll_free=PhoneNumberDesc(national_number_pattern='(?:800|90\\d)\\d{5}', example_number='90123456', possible_length=(8,)),
    no_international_dialling=PhoneNumberDesc(national_number_pattern='8[07]0\\d{5}', possible_length=(8,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{2})', format='\\1 \\2 \\3', leading_digits_pattern=['87']),
        NumberFormat(pattern='(\\d{2})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['4'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[389]']),
        NumberFormat(pattern='(\\d)(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4 \\5', leading_digits_pattern=['[67]'], national_prefix_formatting_rule='0\\1')],
    intl_number_format=[NumberFormat(pattern='(\\d{2})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['4']),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[389]']),
        NumberFormat(pattern='(\\d)(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4 \\5', leading_digits_pattern=['[67]'])])



"""Auto-generated file, do not edit by hand. RO metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_RO = PhoneMetadata(id='RO', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[18]\\d{2,5}', possible_length=(3, 4, 5, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='11(?:[23]|6\\d{3})', example_number='112', possible_length=(3, 6)),
    premium_rate=PhoneNumberDesc(national_number_pattern='(?:1(?:18[39]|[24])|8[48])\\d\\d', example_number='1200', possible_length=(4, 6)),
    emergency=PhoneNumberDesc(national_number_pattern='112', example_number='112', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:1(?:[23]|6(?:000|1(?:11|23))|8(?:(?:01|8[18])1|119|[23]00|932))|[24]\\d\\d|9(?:0(?:00|19)|1[19]|21|3[02]|5[178]))|8[48]\\d\\d', example_number='112', possible_length=(3, 4, 5, 6)),
    sms_services=PhoneNumberDesc(national_number_pattern='113|(?:1[24]|8[48])\\d\\d', example_number='113', possible_length=(3, 4)),
    short_data=True)



from re import Pattern

from .util import U_EMPTY_STRING, U_SPACE
from .phonemetadata import NumberFormat
from .phonemetadata import PhoneMetadata
from .phonemetadata import REGION_CODE_FOR_NON_GEO_ENTITY

_SEPARATOR_BEFORE_NATIONAL_NUMBER: str
_EMPTY_METADATA: PhoneMetadata
_NATIONAL_PREFIX_SEPARATORS_PATTERN: Pattern[str]
_ELIGIBLE_FORMAT_PATTERN: Pattern[str]
_MIN_LEADING_DIGITS_LENGTH: int
_DIGIT_PLACEHOLDER: str
_DIGIT_PATTERN: Pattern[str]

def _get_metadata_for_region(region_code: str) -> PhoneMetadata: ...

class AsYouTypeFormatter:
    _default_country: str
    _current_metadata: PhoneMetadata
    _default_metadata: PhoneMetadata
    _current_output: str
    _accrued_input: str
    _accrued_input_without_formatting: str
    _formatting_template: str
    _last_match_position: int
    _current_formatting_pattern: str
    _prefix_before_national_number: str
    _should_add_space_after_national_prefix: bool
    _extracted_national_prefix: str
    _national_number: str
    _able_to_format: bool
    _input_has_formatting: bool
    _position_to_remember: int
    _original_position: int
    _is_complete_number: bool
    _is_expecting_country_calling_code: bool
    _possible_formats: list[NumberFormat]
    def __init__(self, region_code: str) -> None: ...
    def _maybe_create_new_template(self) -> bool: ...
    def _get_available_formats(self, leading_digits: str) -> None: ...
    def _narrow_down_possible_formats(self, leading_digits: str) -> None: ...
    def _create_formatting_template(self, num_format: NumberFormat) -> bool: ...
    def _get_formatting_template(self, number_pattern: str, number_format: str) -> str: ...
    def _clear(self) -> None: ...
    def clear(self) -> None: ...
    def input_digit(self, next_char: str, remember_position: bool = ...) -> str: ...
    def _attempt_to_choose_pattern_with_prefix_extracted(self) -> str: ...
    def _able_to_extract_longer_ndd(self) -> bool: ...
    def _is_digit_or_leading_plus_sign(self, next_char: str) -> bool: ...
    def _attempt_to_format_accrued_digits(self) -> str: ...
    def get_remembered_position(self) -> int: ...
    def _append_national_number(self, national_number: str) -> str: ...
    def _attempt_to_choose_formatting_pattern(self) -> str: ...
    def _input_accrued_national_number(self) -> str: ...
    def _is_nanpa_number_with_national_prefix(self) -> bool: ...
    def _remove_national_prefix_from_national_number(self) -> str: ...
    def _attempt_to_extract_idd(self) -> bool: ...
    def _attempt_to_extract_ccc(self) -> bool: ...
    def _normalize_and_accrue_digits_and_plus_sign(self, next_char: str, remember_position: bool) -> str: ...
    def _input_digit_helper(self, next_char: str) -> str: ...



"""Auto-generated file, do not edit by hand. FR metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_FR = PhoneMetadata(id='FR', country_code=33, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[1-9]\\d{8}', possible_length=(9,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:26[013-9]|59[1-35-9])\\d{6}|(?:[13]\\d|2[0-57-9]|4[1-9]|5[0-8])\\d{7}', example_number='123456789', possible_length=(9,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:6(?:[0-24-8]\\d|3[0-8]|9[589])|7[3-9]\\d)\\d{6}', example_number='612345678', possible_length=(9,)),
    toll_free=PhoneNumberDesc(national_number_pattern='80[0-5]\\d{6}', example_number='801234567', possible_length=(9,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='836(?:0[0-36-9]|[1-9]\\d)\\d{4}|8(?:1[2-9]|2[2-47-9]|3[0-57-9]|[569]\\d|8[0-35-9])\\d{6}', example_number='891123456', possible_length=(9,)),
    shared_cost=PhoneNumberDesc(national_number_pattern='8(?:1[01]|2[0156]|4[024]|84)\\d{6}', example_number='884012345', possible_length=(9,)),
    voip=PhoneNumberDesc(national_number_pattern='9\\d{8}', example_number='912345678', possible_length=(9,)),
    uan=PhoneNumberDesc(national_number_pattern='80[6-9]\\d{6}', example_number='806123456', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{4})', format='\\1', leading_digits_pattern=['10']),
        NumberFormat(pattern='(\\d{3})(\\d{3})', format='\\1 \\2', leading_digits_pattern=['1']),
        NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['8'], national_prefix_formatting_rule='0 \\1'),
        NumberFormat(pattern='(\\d)(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4 \\5', leading_digits_pattern=['[1-79]'], national_prefix_formatting_rule='0\\1')],
    intl_number_format=[NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['8']),
        NumberFormat(pattern='(\\d)(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4 \\5', leading_digits_pattern=['[1-79]'])],
    mobile_number_portable_region=True)



"""Auto-generated file, do not edit by hand. QA metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_QA = PhoneMetadata(id='QA', country_code=974, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='800\\d{4}|(?:2|800)\\d{6}|(?:0080|[3-7])\\d{7}', possible_length=(7, 8, 9, 11)),
    fixed_line=PhoneNumberDesc(national_number_pattern='4(?:(?:[014]\\d\\d|999)\\d|2022)\\d{3}', example_number='44123456', possible_length=(8,)),
    mobile=PhoneNumberDesc(national_number_pattern='[35-7]\\d{7}', example_number='33123456', possible_length=(8,)),
    toll_free=PhoneNumberDesc(national_number_pattern='800\\d{4}|(?:0080[01]|800)\\d{6}', example_number='8001234', possible_length=(7, 9, 11)),
    pager=PhoneNumberDesc(national_number_pattern='2[136]\\d{5}', example_number='2123456', possible_length=(7,)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{4})', format='\\1 \\2', leading_digits_pattern=['2[136]|8']),
        NumberFormat(pattern='(\\d{4})(\\d{4})', format='\\1 \\2', leading_digits_pattern=['[3-7]'])],
    mobile_number_portable_region=True)



"""Auto-generated file, do not edit by hand. PL metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_PL = PhoneMetadata(id='PL', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[19]\\d\\d(?:\\d{2,3})?', possible_length=(3, 5, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='11(?:2|6\\d{3})|99[7-9]', example_number='112', possible_length=(3, 6)),
    emergency=PhoneNumberDesc(national_number_pattern='112|99[7-9]', example_number='112', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:1(?:2|61(?:11|23)|891[23])|9\\d{3})|9(?:8[4-7]|9[1-9])|11[68]000', example_number='112', possible_length=(3, 5, 6)),
    short_data=True)



"""Auto-generated file, do not edit by hand. SB metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_SB = PhoneMetadata(id='SB', country_code=677, international_prefix='0[01]',
    general_desc=PhoneNumberDesc(national_number_pattern='[6-9]\\d{6}|[1-6]\\d{4}', possible_length=(5, 7)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:1[4-79]|[23]\\d|4[0-2]|5[03]|6[0-37])\\d{3}', example_number='40123', possible_length=(5,)),
    mobile=PhoneNumberDesc(national_number_pattern='48\\d{3}|(?:(?:6[89]|7[1-9]|8[4-9])\\d|9(?:1[2-9]|2[013-9]|3[0-2]|[46]\\d|5[0-46-9]|7[0-689]|8[0-79]|9[0-8]))\\d{4}', example_number='7421234', possible_length=(5, 7)),
    toll_free=PhoneNumberDesc(national_number_pattern='1[38]\\d{3}', example_number='18123', possible_length=(5,)),
    voip=PhoneNumberDesc(national_number_pattern='5[12]\\d{3}', example_number='51123', possible_length=(5,)),
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{5})', format='\\1 \\2', leading_digits_pattern=['6[89]|7|8[4-9]|9(?:[1-8]|9[0-8])'])])



"""Auto-generated file, do not edit by hand. WF metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_WF = PhoneMetadata(id='WF', country_code=681, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:40|72|8\\d{4})\\d{4}|[89]\\d{5}', possible_length=(6, 9)),
    fixed_line=PhoneNumberDesc(national_number_pattern='72\\d{4}', example_number='721234', possible_length=(6,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:72|8[23])\\d{4}', example_number='821234', possible_length=(6,)),
    toll_free=PhoneNumberDesc(national_number_pattern='80[0-5]\\d{6}', example_number='800012345', possible_length=(9,)),
    voip=PhoneNumberDesc(national_number_pattern='9[23]\\d{4}', example_number='921234', possible_length=(6,)),
    voicemail=PhoneNumberDesc(national_number_pattern='[48]0\\d{4}', example_number='401234', possible_length=(6,)),
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3', leading_digits_pattern=['[47-9]']),
        NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['8'])])



from .phonenumber import PhoneNumber

__all__: list[str]

def name_for_valid_number(numobj: PhoneNumber, lang: str, script: str | None = ..., region: str | None = ...) -> str: ...
def name_for_number(numobj: PhoneNumber, lang: str, script: str | None = ..., region: str | None = ...) -> str: ...
def safe_display_name(numobj: PhoneNumber, lang: str, script: str | None = ..., region: str | None = ...) -> str: ...
def _is_mobile(ntype: int) -> bool: ...



"""Auto-generated file, do not edit by hand. 66 metadata"""
from ..phonemetadata import NumberFormat

PHONE_ALT_FORMAT_66 = [NumberFormat(pattern='(\\d{4})(\\d{4})', format='\\1 \\2', leading_digits_pattern=['2'])]



"""Auto-generated file, do not edit by hand. ID metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_ID = PhoneMetadata(id='ID', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[178]\\d\\d(?:\\d{2,3})?', possible_length=(3, 5, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='11[02389]', example_number='110', possible_length=(3,)),
    emergency=PhoneNumberDesc(national_number_pattern='11[02389]', example_number='110', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:1[02389]|40\\d\\d|50264)|71400|89887', example_number='110', possible_length=(3, 5, 6)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='(?:714|898)\\d\\d', example_number='71400', possible_length=(5,)),
    sms_services=PhoneNumberDesc(national_number_pattern='714\\d\\d', example_number='71400', possible_length=(5,)),
    short_data=True)



"""Python 2.x/3.x compatibility utilities.

This module defines a collection of functions that allow the same Python
source code to be used in both Python 2.x and Python 3.x.

 - prnt() prints its arguments to a file, with given separator and ending.
 - to_long() creates a (long) integer object from its input parameter.
 - u() allows string literals involving non-ASCII characters to be
   used in both Python 2.x / 3.x, e.g. u("\u0101 is a-with-macron")
 - unicod() forces its argument to a Unicode string.
 - rpr() generates a representation of a string that can be parsed in either
   Python 2.x or 3.x, assuming use of the u() function above.

>>> from .util import prnt, u, rpr
>>> prnt("hello")
hello
>>> prnt("hello", "world")
hello world
>>> prnt("hello", "world", sep=":")
hello:world
>>> prnt("hello", "world", sep=":", end='!\\n')
hello:world!
>>> u('\u0101') == u('\U00000101')
True
>>> u('\u0101') == u('\N{LATIN SMALL LETTER A WITH MACRON}')
True
>>> a_macron = u('\u0101')
>>> rpr(a_macron)
"u('\\\\u0101')"
>>> rpr(u('abc')) == "'abc'"  # In Python 2, LHS is Unicode but RHS is string
True
>>> rpr("'")
"'\\\\''"
"""
import sys


if sys.version_info >= (3, 0):  # pragma no cover
    import builtins
    print3 = builtins.__dict__['print']

    unicod = str
    u = str
    to_long = int

    def prnt(*args, **kwargs):
        sep = kwargs.get('sep', ' ')
        end = kwargs.get('end', '\n')
        file = kwargs.get('file', None)
        print3(*args, **{'sep': sep, 'end': end, 'file': file})

    class UnicodeMixin(object):
        """Mixin class to define a __str__ method in terms of __unicode__ method"""
        def __str__(self):
            return self.__unicode__()

else:  # pragma no cover
    unicod = unicode

    import unicodedata
    import re
    # \N{name} = character named name in the Unicode database
    _UNAME_RE = re.compile(r'\\N\{(?P<name>[^}]+)\}')
    # \uxxxx = character with 16-bit hex value xxxx
    _U16_RE = re.compile(r'\\u(?P<hexval>[0-9a-fA-F]{4})')
    # \Uxxxxxxxx = character with 32-bit hex value xxxxxxxx
    _U32_RE = re.compile(r'\\U(?P<hexval>[0-9a-fA-F]{8})')

    def u(s):
        """Generate Unicode string from a string input, encoding Unicode characters.

        This is expected to work in the same way as u'<string>' would work in Python
        2.x (although it is not completely robust as it is based on a simple set of
        regexps).
        """
        us = re.sub(_U16_RE, lambda m: unichr(int(m.group('hexval'), 16)), unicode(s))
        us = re.sub(_U32_RE, lambda m: unichr(int(m.group('hexval'), 16)), us)
        us = re.sub(_UNAME_RE, lambda m: unicodedata.lookup(m.group('name')), us)
        return us

    to_long = long

    def prnt(*args, **kwargs):
        sep = kwargs.get('sep', ' ')
        end = kwargs.get('end', '\n')
        file = kwargs.get('file', None)
        if file is None:
            file = sys.stdout
        print >> file, sep.join([str(arg) for arg in args]) + end,

    class UnicodeMixin(object):  # pragma no cover
        """Mixin class to define a __str__ method in terms of __unicode__ method"""
        def __str__(self):
            return unicode(self).encode('utf-8')

# Constants for Unicode strings
U_EMPTY_STRING = unicod("")
U_SPACE = unicod(" ")
U_DASH = unicod("-")
U_TILDE = unicod("~")
U_PLUS = unicod("+")
U_STAR = unicod("*")
U_ZERO = unicod("0")
U_SLASH = unicod("/")
U_SEMICOLON = unicod(";")
U_X_LOWER = unicod("x")
U_X_UPPER = unicod("X")
U_PERCENT = unicod("%")


def rpr(s):
    """Create a representation of a Unicode string that can be used in both
    Python 2 and Python 3k, allowing for use of the u() function"""
    if s is None:
        return 'None'
    seen_unicode = False
    results = []
    for cc in s:
        ccn = ord(cc)
        if ccn >= 32 and ccn < 127:
            if cc == "'":  # escape single quote
                results.append('\\')
                results.append(cc)
            elif cc == "\\":  # escape backslash
                results.append('\\')
                results.append(cc)
            else:
                results.append(cc)
        else:
            seen_unicode = True
            if ccn <= 0xFFFF:
                results.append('\\u')
                results.append("%04x" % ccn)
            else:  # pragma no cover
                results.append('\\U')
                results.append("%08x" % ccn)
    result = "'" + "".join(results) + "'"
    if seen_unicode:
        return "u(" + result + ")"
    else:
        return result


def force_unicode(s):
    """Force the argument to be a Unicode string, preserving None"""
    if s is None:
        return None
    else:
        return unicod(s)


class ImmutableMixin(object):
    """Mixin class to make objects of subclasses immutable"""
    _mutable = False

    def __setattr__(self, name, value):
        if self._mutable or name == "_mutable":
            object.__setattr__(self, name, value)
        else:
            raise TypeError("Can't modify immutable instance")

    def __delattr__(self, name):
        if self._mutable:
            object.__delattr__(self, name)
        else:
            raise TypeError("Can't modify immutable instance")


def mutating_method(func):
    """Decorator for methods that are allowed to modify immutable objects"""
    def wrapper(self, *__args, **__kwargs):
        old_mutable = self._mutable
        self._mutable = True
        try:
            # Call the wrapped function
            return func(self, *__args, **__kwargs)
        finally:
            self._mutable = old_mutable
    return wrapper


if __name__ == '__main__':  # pragma no cover
    import doctest
    doctest.testmod()



"""Auto-generated file, do not edit by hand. MQ metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_MQ = PhoneMetadata(id='MQ', country_code=596, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='7091\\d{5}|(?:[56]9|[89]\\d)\\d{7}', possible_length=(9,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:59(?:6(?:[03-7]\\d|1[05]|2[7-9]|8[0-39]|9[04-9])|89\\d)|80[6-9]\\d\\d|9(?:477[6-9]|767[4589]))\\d{4}', example_number='596301234', possible_length=(9,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:69[67]\\d\\d|7091[0-3])\\d{4}', example_number='696201234', possible_length=(9,)),
    toll_free=PhoneNumberDesc(national_number_pattern='80[0-5]\\d{6}', example_number='800012345', possible_length=(9,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='8[129]\\d{7}', example_number='810123456', possible_length=(9,)),
    voip=PhoneNumberDesc(national_number_pattern='9(?:397[0-3]|477[0-5]|76(?:6\\d|7[0-367]))\\d{4}', example_number='976612345', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[5-79]|8(?:0[6-9]|[36])'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['8'], national_prefix_formatting_rule='0\\1')],
    mobile_number_portable_region=True)



"""Auto-generated file, do not edit by hand. NE metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_NE = PhoneMetadata(id='NE', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[1-3578]\\d(?:\\d(?:\\d{3})?)?', possible_length=(2, 3, 6)),
    toll_free=PhoneNumberDesc(national_number_pattern='1(?:18|[578])|723\\d{3}', example_number='15', possible_length=(2, 3, 6)),
    emergency=PhoneNumberDesc(national_number_pattern='1(?:18|[578])|723141', example_number='15', possible_length=(2, 3, 6)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:0[01]|1[128]|2[034]|3[013]|[46]0|55?|[78])|222|333|555|723141|888', example_number='15', possible_length=(2, 3, 6)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='1(?:0[01]|1[12]|2[034]|3[013]|[46]0|55)|222|333|555|888', example_number='100', possible_length=(3,)),
    short_data=True)



"""Auto-generated file, do not edit by hand. WS metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_WS = PhoneMetadata(id='WS', country_code=None, international_prefix=None,
    general_desc=PhoneNumberDesc(national_number_pattern='[19]\\d\\d', possible_length=(3,)),
    toll_free=PhoneNumberDesc(national_number_pattern='9(?:11|9[4-69])', example_number='911', possible_length=(3,)),
    emergency=PhoneNumberDesc(national_number_pattern='9(?:11|9[4-69])', example_number='911', possible_length=(3,)),
    short_code=PhoneNumberDesc(national_number_pattern='1(?:1[12]|2[0-6]|[39]0)|9(?:11|9[4-79])', example_number='111', possible_length=(3,)),
    carrier_specific=PhoneNumberDesc(national_number_pattern='12[0-6]', example_number='120', possible_length=(3,)),
    short_data=True)



"""Auto-generated file, do not edit by hand. MA metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_MA = PhoneMetadata(id='MA', country_code=212, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[5-8]\\d{8}', possible_length=(9,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='5(?:(?:18|4[0679]|5[03])\\d|2(?:[0-25-79]\\d|3[1-578]|4[02-46-8]|8[0235-9])|3(?:[0-47]\\d|5[02-9]|6[02-8]|8[014-9]|9[3-9]))\\d{5}', example_number='520123456', possible_length=(9,)),
    mobile=PhoneNumberDesc(national_number_pattern='(?:6(?:[0-79]\\d|8[0-247-9])|7(?:[016-8]\\d|2[0-8]|5[0-5]))\\d{6}', example_number='650123456', possible_length=(9,)),
    toll_free=PhoneNumberDesc(national_number_pattern='80[0-7]\\d{6}', example_number='801234567', possible_length=(9,)),
    premium_rate=PhoneNumberDesc(national_number_pattern='89\\d{7}', example_number='891234567', possible_length=(9,)),
    voip=PhoneNumberDesc(national_number_pattern='(?:592(?:4[0-2]|93)|80[89]\\d\\d)\\d{4}', example_number='592401234', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{4})(\\d{5})', format='\\1-\\2', leading_digits_pattern=['892'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{2})(\\d{7})', format='\\1-\\2', leading_digits_pattern=['8(?:0[0-7]|9)'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d)(\\d{2})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4 \\5', leading_digits_pattern=['[5-8]'], national_prefix_formatting_rule='0\\1')],
    main_country_for_code=True,
    leading_digits='[5-8]',
    mobile_number_portable_region=True)



"""Auto-generated file, do not edit by hand. AE metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_AE = PhoneMetadata(id='AE', country_code=971, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='(?:[4-7]\\d|9[0-689])\\d{7}|800\\d{2,9}|[2-4679]\\d{7}', possible_length=(5, 6, 7, 8, 9, 10, 11, 12)),
    fixed_line=PhoneNumberDesc(national_number_pattern='[2-4679][2-8]\\d{6}', example_number='22345678', possible_length=(8,), possible_length_local_only=(7,)),
    mobile=PhoneNumberDesc(national_number_pattern='5[02-68]\\d{7}', example_number='501234567', possible_length=(9,)),
    toll_free=PhoneNumberDesc(national_number_pattern='400\\d{6}|800\\d{2,9}', example_number='800123456', possible_length=(5, 6, 7, 8, 9, 10, 11, 12)),
    premium_rate=PhoneNumberDesc(national_number_pattern='900[02]\\d{5}', example_number='900234567', possible_length=(9,)),
    shared_cost=PhoneNumberDesc(national_number_pattern='700[05]\\d{5}', example_number='700012345', possible_length=(9,)),
    uan=PhoneNumberDesc(national_number_pattern='600[25]\\d{5}', example_number='600212345', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{2,9})', format='\\1 \\2', leading_digits_pattern=['60|8']),
        NumberFormat(pattern='(\\d)(\\d{3})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['[236]|[479][2-8]'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{3})(\\d)(\\d{5})', format='\\1 \\2 \\3', leading_digits_pattern=['[479]']),
        NumberFormat(pattern='(\\d{2})(\\d{3})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['5'], national_prefix_formatting_rule='0\\1')])



"""PhoneNumber object definition"""

# Based on original Java code and protocol buffer:
#     resources/phonenumber.proto
#     java/src/com/google/i18n/phonenumbers/Phonenumber.java
#   Copyright (C) 2010-2011 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from .util import UnicodeMixin, ImmutableMixin, mutating_method
from .util import to_long, unicod, rpr, force_unicode, u


class CountryCodeSource(object):
    """The source from which a country code is derived."""
    # Default value returned if this is not set, because the phone number was
    # created using parse(keep_raw_input=False).
    UNSPECIFIED = 0

    # The country_code is derived based on a phone number with a leading "+",
    # e.g. the French number "+33 1 42 68 53 00".
    FROM_NUMBER_WITH_PLUS_SIGN = 1

    # The country_code is derived based on a phone number with a leading IDD,
    # e.g. the French number "011 33 1 42 68 53 00", as it is dialled
    # from US.
    FROM_NUMBER_WITH_IDD = 5

    # The country_code is derived based on a phone number without a leading
    # "+", e.g. the French number "33 1 42 68 53 00" when default_country is
    # supplied as France.
    FROM_NUMBER_WITHOUT_PLUS_SIGN = 10

    # The country_code is derived NOT based on the phone number itself, but
    # from the default_country parameter provided in the parsing function by
    # the clients. This happens mostly for numbers written in the national
    # format (without country code). For example, this would be set when
    # parsing the French number "01 42 68 53 00", when default_country is
    # supplied as France.
    FROM_DEFAULT_COUNTRY = 20

    @classmethod
    def to_string(cls, val):
        """Return a string representation of a CountryCodeSource value"""
        if val == CountryCodeSource.UNSPECIFIED:
            return u("UNSPECIFIED")
        elif val == CountryCodeSource.FROM_NUMBER_WITH_PLUS_SIGN:
            return u("FROM_NUMBER_WITH_PLUS_SIGN")
        elif val == CountryCodeSource.FROM_NUMBER_WITH_IDD:
            return u("FROM_NUMBER_WITH_IDD")
        elif val == CountryCodeSource.FROM_NUMBER_WITHOUT_PLUS_SIGN:
            return u("FROM_NUMBER_WITHOUT_PLUS_SIGN")
        elif val == CountryCodeSource.FROM_DEFAULT_COUNTRY:
            return u("FROM_DEFAULT_COUNTRY")
        else:
            return u("INVALID (%d)" % val)


class PhoneNumber(UnicodeMixin):
    """Class representing international telephone numbers.

    This class is hand-created based on phonenumber.proto. Please refer
    to that file for detailed descriptions of the meaning of each field.
    """

    def __init__(self,
                 country_code=None,
                 national_number=None,
                 extension=None,
                 italian_leading_zero=None,
                 number_of_leading_zeros=None,
                 raw_input=None,
                 country_code_source=CountryCodeSource.UNSPECIFIED,
                 preferred_domestic_carrier_code=None):
        # The country calling code for this number, as defined by the
        # International Telecommunication Union (ITU). For example, this would
        # be 1 for NANPA countries, and 33 for France.
        #
        # None if not set, of type int otherwise.
        if country_code is None:
            self.country_code = None
        else:
            self.country_code = int(country_code)

        # Number does not contain National(trunk) prefix.
        # National (significant) Number is defined in International
        # Telecommunication Union (ITU) Recommendation E.164. It is a
        # language/country-neutral representation of a phone number at a
        # country level. For countries which have the concept of an "area
        # code" or "national destination code", this is included in the
        # National (significant) Number. Although the ITU says the maximum
        # length should be 15, we have found longer numbers in some countries
        # e.g. Germany.  Note that the National (significant) Number does not
        # contain the National(trunk) prefix.
        #
        # None if not set, of type long otherwise (and so it will never
        # contain any formatting (hypens, spaces, parentheses), nor any
        # alphanumeric spellings).

        if national_number is None:
            self.national_number = None
        else:
            self.national_number = to_long(national_number)

        # Extension is not standardized in ITU recommendations, except for
        # being defined as a series of numbers with a maximum length of 40
        # digits.
        #
        # When present, it is a Unicode string to accommodate for the
        # possible use of a leading zero in the extension (organizations
        # have complete freedom to do so, as there is no standard defined).
        # However, only ASCII digits should be stored here.
        self.extension = force_unicode(extension)  # None or Unicode '[0-9]+'

        # In some countries, the national (significant) number starts with one
        # or more "0"s without this being a national prefix or trunk code of
        # some kind. For example, the leading zero in the national
        # (significant) number of an Italian phone number indicates the number
        # is a fixed-line number.  There have been plans to migrate fixed-line
        # numbers to start with the digit two since December 2000, but it has
        # not happened yet. See http://en.wikipedia.org/wiki/%2B39 for more
        # details.
        #
        # These fields can be safely ignored (there is no need to set them)
        # for most countries. Some limited number of countries behave like
        # Italy - for these cases, if the leading zero(s) of a number would be
        # retained even when dialling internationally, set this flag to true,
        # and also set the number of leading zeros.
        #
        # Clients who use the parsing functionality of the i18n phone number
        # libraries will have these fields set if necessary automatically.
        #
        # None if not set, of type bool otherwise:
        if italian_leading_zero is None:
            self.italian_leading_zero = None
        else:
            self.italian_leading_zero = bool(italian_leading_zero)

        # None if not set, of type int otherwise.
        if number_of_leading_zeros is None:
            self.number_of_leading_zeros = None
        else:
            self.number_of_leading_zeros = int(number_of_leading_zeros)

        # The next few fields are non-essential fields for a phone number.
        # They retain extra information about the form the phone number was
        # in when it was provided to us to parse. They can be safely
        # ignored by most clients.

        # This field is used to store the raw input string containing phone
        # numbers before it was canonicalized by the library. For example, it
        # could be used to store alphanumerical numbers such as
        # "1-800-GOOG-411".
        self.raw_input = force_unicode(raw_input)  # None or Unicode string

        # The source from which the country_code is derived. This is not set
        # in the general parsing method, but in the method that parses and
        # keeps raw_input. New fields could be added upon request.
        self.country_code_source = country_code_source  # CountryCodeSource.VALUE
        if self.country_code_source is None:  # pragma no cover
            self.country_code_source = CountryCodeSource.UNSPECIFIED

        # The carrier selection code that is preferred when calling this
        # phone number domestically. This also includes codes that need to
        # be dialed in some countries when calling from landlines to mobiles
        # or vice versa. For example, in Columbia, a "3" needs to be dialed
        # before the phone number itself when calling from a mobile phone to
        # a domestic landline phone and vice versa.
        #
        # Note this is the "preferred" code, which means other codes may work
        # as well.
        self.preferred_domestic_carrier_code = force_unicode(preferred_domestic_carrier_code)
        # None or Unicode string

    def clear(self):
        """Erase the contents of the object"""
        self.country_code = None
        self.national_number = None
        self.extension = None
        self.italian_leading_zero = None
        self.number_of_leading_zeros = None
        self.raw_input = None
        self.country_code_source = CountryCodeSource.UNSPECIFIED
        self.preferred_domestic_carrier_code = None

    def merge_from(self, other):
        """Merge information from another PhoneNumber object into this one."""
        if other.country_code is not None:
            self.country_code = other.country_code
        if other.national_number is not None:
            self.national_number = other.national_number
        if other.extension is not None:
            self.extension = other.extension
        if other.italian_leading_zero is not None:
            self.italian_leading_zero = other.italian_leading_zero
        if other.number_of_leading_zeros is not None:
            self.number_of_leading_zeros = other.number_of_leading_zeros
        if other.raw_input is not None:
            self.raw_input = other.raw_input
        if other.country_code_source is not CountryCodeSource.UNSPECIFIED:
            self.country_code_source = other.country_code_source
        if other.preferred_domestic_carrier_code is not None:
            self.preferred_domestic_carrier_code = other.preferred_domestic_carrier_code

    def __eq__(self, other):
        if not isinstance(other, PhoneNumber):
            return False
        return (self.country_code == other.country_code and
                self.national_number == other.national_number and
                self.extension == other.extension and
                bool(self.italian_leading_zero) == bool(other.italian_leading_zero) and
                self.number_of_leading_zeros == other.number_of_leading_zeros and
                self.raw_input == other.raw_input and
                self.country_code_source == other.country_code_source and
                self.preferred_domestic_carrier_code == other.preferred_domestic_carrier_code)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return (unicod("%s(country_code=%s, national_number=%s, extension=%s, " +
                       "italian_leading_zero=%s, number_of_leading_zeros=%s, " +
                       "country_code_source=%s, preferred_domestic_carrier_code=%s)") %
                (type(self).__name__,
                 self.country_code,
                 self.national_number,
                 rpr(self.extension),
                 self.italian_leading_zero,
                 self.number_of_leading_zeros,
                 self.country_code_source,
                 rpr(self.preferred_domestic_carrier_code)))

    def __unicode__(self):
        result = (unicod("Country Code: %s National Number: %s") %
                  (self.country_code, self.national_number))
        if self.italian_leading_zero is not None:
            result += unicod(" Leading Zero(s): %s") % self.italian_leading_zero
        if self.number_of_leading_zeros is not None:
            result += unicod(" Number of leading zeros: %d") % self.number_of_leading_zeros
        if self.extension is not None:
            result += unicod(" Extension: %s") % self.extension
        if self.country_code_source is not CountryCodeSource.UNSPECIFIED:
            result += unicod(" Country Code Source: %s") % self.country_code_source
        if self.preferred_domestic_carrier_code is not None:
            result += (unicod(" Preferred Domestic Carrier Code: %s") %
                       self.preferred_domestic_carrier_code)
        return result


class FrozenPhoneNumber(PhoneNumber, ImmutableMixin):
    """Immutable version of PhoneNumber"""
    def __hash__(self):
        return hash((self.country_code,
                     self.national_number,
                     self.extension,
                     bool(self.italian_leading_zero),
                     self.number_of_leading_zeros,
                     self.raw_input,
                     self.country_code_source,
                     self.preferred_domestic_carrier_code))

    @mutating_method
    def __init__(self, *args, **kwargs):
        if len(kwargs) == 0 and len(args) == 1 and isinstance(args[0], PhoneNumber):
            # Copy constructor
            super(FrozenPhoneNumber, self).__init__(**args[0].__dict__)
        else:
            super(FrozenPhoneNumber, self).__init__(*args, **kwargs)



"""Auto-generated file, do not edit by hand. 84 metadata"""
from ..phonemetadata import NumberFormat

PHONE_ALT_FORMAT_84 = [NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['6']), NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{3})(\\d{3})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['2[48]']), NumberFormat(pattern='(\\d{4})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['1'])]



from .phonenumber import PhoneNumber

__all__: list[str]
UNKNOWN_TIMEZONE: str
_UNKNOWN_TIME_ZONE_LIST: tuple[str, ...]

def time_zones_for_geographical_number(numobj: PhoneNumber) -> tuple[str, ...]: ...
def time_zones_for_number(numobj: PhoneNumber) -> tuple[str, ...]: ...
def _country_level_time_zones_for_number(numobj: PhoneNumber) -> tuple[str, ...]: ...



"""Auto-generated file, do not edit by hand. AO metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_AO = PhoneMetadata(id='AO', country_code=244, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[29]\\d{8}', possible_length=(9,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='2\\d(?:[0134][25-9]|[25-9]\\d)\\d{5}', example_number='222123456', possible_length=(9,)),
    mobile=PhoneNumberDesc(national_number_pattern='9[1-79]\\d{7}', example_number='923123456', possible_length=(9,)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3})', format='\\1 \\2 \\3', leading_digits_pattern=['[29]'])])



"""Auto-generated file, do not edit by hand. GM metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_GM = PhoneMetadata(id='GM', country_code=220, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[2-9]\\d{6}', possible_length=(7,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='(?:4(?:[23]\\d\\d|4(?:1[024679]|[6-9]\\d))|5(?:5(?:3\\d|4[0-7])|6[67]\\d|7(?:1[04]|2[035]|3[58]|48))|8[0-589]\\d\\d)\\d{3}', example_number='5661234', possible_length=(7,)),
    mobile=PhoneNumberDesc(national_number_pattern='556\\d{4}|(?:[23679]\\d|4[015]|5[0-489]|8[67])\\d{5}', example_number='3012345', possible_length=(7,)),
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{4})', format='\\1 \\2', leading_digits_pattern=['[2-9]'])])
